@@ -72,7 +72,7 @@ class AKONADIPROTOCOLINTERNALS_EXPORT ImapInterval
     /**
       Assignment operator.
     */
-    ImapInterval &operator=( const ImapInterval &other );
+    ImapInterval& operator=( const ImapInterval &other );
 
     /**
       Comparison operator.
@@ -158,27 +158,16 @@ class AKONADIPROTOCOLINTERNALS_EXPORT ImapSet
     /**
       Assignment operator.
     */
-    ImapSet &operator=( const ImapSet &other );
+    ImapSet& operator=( const ImapSet &other );
 
     /**
       Adds the given list of positive integer numbers to the set.
-      The list is sorted and split into as large as possible intervals.
+      The list is sorted and splitted into as large as possible intervals.
       No interval merging is performed.
       @param values List of positive integer numbers in arbitrary order
     */
-    void add( const QVector<Id> &values );
-
-    /**
-       @overload
-       @deprecated Use the QVector version instead.
-    */
     void add( const QList<Id> &values );
 
-    /**
-     * @overload
-     */
-    void add( const QSet<Id> &values );
-
     /**
       Adds the given ImapInterval to this set.
       No interval merging is performed.
@@ -207,11 +196,7 @@ class AKONADIPROTOCOLINTERNALS_EXPORT ImapSet
 
 }
 
-AKONADIPROTOCOLINTERNALS_EXPORT QDebug &operator<<( QDebug &d, const Akonadi::ImapInterval &interval );
-AKONADIPROTOCOLINTERNALS_EXPORT QDebug operator<<( QDebug d, const Akonadi::ImapSet &set );
-
-Q_DECLARE_TYPEINFO( Akonadi::ImapInterval, Q_MOVABLE_TYPE );
-Q_DECLARE_TYPEINFO( Akonadi::ImapSet,      Q_MOVABLE_TYPE );
+AKONADIPROTOCOLINTERNALS_EXPORT QDebug& operator<<( QDebug& d, const Akonadi::ImapInterval &interval );
 
 Q_DECLARE_METATYPE( Akonadi::ImapInterval )
 Q_DECLARE_METATYPE( Akonadi::ImapInterval::List )
