@@ -33,12 +33,16 @@
 #define AKONADI_DBUS_CONTROL_SERVICE "org.freedesktop.Akonadi.Control"
 
 // Commands
+#define AKONADI_CMD_ITEMDELETE "REMOVE"
 #define AKONADI_CMD_ITEMFETCH "FETCH"
 #define AKONADI_CMD_UID "UID"
+#define AKONADI_CMD_RESOURCESELECT "RESSELECT"
+#define AKONADI_CMD_RID "RID"
 
 // Command parameters
 #define AKONADI_PARAM_FULLPAYLOAD "FULLPAYLOAD"
 #define AKONADI_PARAM_ALLATTRIBUTES "ALLATTR"
 #define AKONADI_PARAM_CACHEONLY "CACHEONLY"
+#define AKONADI_PARAM_EXTERNALPAYLOAD "EXTERNALPAYLOAD"
 
 #endif
