@@ -27,7 +27,10 @@
 #include "response.h"
 #include "tracer.h"
 
-#include "../libs/imapparser_p.h"
+#include "libs/imapparser_p.h"
+#include "imapstreamparser.h"
+#include "shared/akdebug.h"
+#include "shared/akcrash.h"
 
 #include <assert.h>
 
@@ -41,6 +44,8 @@ AkonadiConnection::AkonadiConnection( quintptr socketDescriptor, QObject *parent
     , m_connectionState( NonAuthenticated )
     , m_backend( 0 )
     , m_selectedConnection( 0 )
+    , m_parser( 0 )
+    , m_streamParser( 0 )
 {
     m_identifier.sprintf( "%p", static_cast<void*>( this ) );
     Tracer::self()->beginConnection( m_identifier, QString() );
@@ -70,6 +75,8 @@ void AkonadiConnection::run()
         qWarning() << "AkonadiConnection(" << m_identifier
                  << ")::run: failed to set socket descriptor: "
                  << m_socket->error() << "(" << m_socket->errorString() << ")";
+        delete m_socket;
+        m_socket = 0;
         return;
     }
 
@@ -88,11 +95,14 @@ void AkonadiConnection::run()
     connect( m_socket, SIGNAL( disconnected() ),
              this, SLOT( slotDisconnected() ), Qt::DirectConnection );
 
-    writeOut( "* OK Akonadi Almost IMAP Server [PROTOCOL 6]");
+    writeOut( "* OK Akonadi Almost IMAP Server [PROTOCOL 10]");
 
+    m_streamParser = new ImapStreamParser( m_socket );
     exec();
     delete m_socket;
     m_socket = 0;
+    delete m_streamParser;
+    m_streamParser = 0;
 }
 
 void AkonadiConnection::slotDisconnected()
@@ -102,52 +112,48 @@ void AkonadiConnection::slotDisconnected()
 
 void AkonadiConnection::slotNewData()
 {
-  while ( m_socket->bytesAvailable() > 0 ) {
-    if ( m_parser->continuationSize() > 1 ) {
-      const QByteArray data = m_socket->read( qMin( m_socket->bytesAvailable(), m_parser->continuationSize() - 1 ) );
-      Tracer::self()->connectionInput( m_identifier, QLatin1String("[binary data]") );
-      m_parser->parseBlock( data );
-    } else if ( m_socket->canReadLine() ) {
-      const QByteArray line = m_socket->readLine();
-      Tracer::self()->connectionInput( m_identifier, QString::fromUtf8( line ) );
-
-      if ( m_parser->parseNextLine( line ) ) {
-        // parse the command
-        QByteArray command;
-        ImapParser::parseString( m_parser->data(), command );
-
-        m_currentHandler = findHandlerForCommand( command );
-        m_currentHandler->setTag( m_parser->tag() );
-        assert( m_currentHandler );
-        connect( m_currentHandler, SIGNAL( responseAvailable( const Response & ) ),
-                this, SLOT( slotResponseAvailable( const Response & ) ), Qt::DirectConnection );
-        connect( m_currentHandler, SIGNAL( connectionStateChange( ConnectionState ) ),
-                this, SLOT( slotConnectionStateChange( ConnectionState ) ),
-                Qt::DirectConnection );
-        try {
-            // FIXME: remove the tag, it's only there for backward compatibility with the handlers!
-            if ( m_currentHandler->handleLine( m_parser->tag() + ' ' + m_parser->data() ) )
-                m_currentHandler = 0;
-        } catch ( ... ) {
-            delete m_currentHandler;
-            m_currentHandler = 0;
-        }
-        m_parser->reset();
-      } else {
-        if ( m_parser->continuationStarted() ) {
-          Response response;
-          response.setContinuation();
-          response.setString( "Ready for literal data (expecting " +
-              QByteArray::number( m_parser->continuationSize() ) + " bytes)" );
-          slotResponseAvailable( response );
-        }
+  while ( m_socket->bytesAvailable() > 0 || !m_streamParser->readRemainingData().isEmpty()) {
+    QByteArray tag = m_streamParser->readString();
+    QByteArray command = m_streamParser->readString();
+    m_currentHandler = findHandlerForCommand( command );
+    m_currentHandler->setTag( tag );
+    Tracer::self()->connectionInput( m_identifier, QString::fromUtf8( tag + " " + command + " " + m_streamParser->readRemainingData() ) );
+    assert( m_currentHandler );
+    connect( m_currentHandler, SIGNAL( responseAvailable( const Response & ) ),
+             this, SLOT( slotResponseAvailable( const Response & ) ), Qt::DirectConnection );
+    connect( m_currentHandler, SIGNAL( connectionStateChange( ConnectionState ) ),
+             this, SLOT( slotConnectionStateChange( ConnectionState ) ),
+                         Qt::DirectConnection );
+    try {
+      m_currentHandler->setStreamParser( m_streamParser );
+      if ( !m_currentHandler->parseStream() ) {
+        m_streamParser->readUntilCommandEnd(); //just eat the ending newline
       }
-    } else {
-      break; // nothing we can do for now with the available data
+      m_currentHandler = 0;
+    } catch ( const Akonadi::HandlerException &e ) {
+      m_currentHandler->failureResponse( e.what() );
+      m_currentHandler->deleteLater();
+      m_currentHandler = 0;
+      m_streamParser->readUntilCommandEnd(); //just eat the ending newline
+    } catch ( const Akonadi::Exception &e ) {
+      m_currentHandler->failureResponse( QString::fromLatin1( e.type() )
+          + QLatin1String( ": " ) + QString::fromLatin1( e.what()  ) );
+      m_currentHandler->deleteLater();
+      m_currentHandler = 0;
+      m_streamParser->readUntilCommandEnd(); //just eat the ending newline
+    } catch ( ... ) {
+      akError() << "Unknown exception caught: " << akBacktrace();
+      delete m_currentHandler;
+      m_currentHandler = 0;
+      m_streamParser->readUntilCommandEnd(); //just eat the ending newline
     }
+
+    if (m_streamParser->readRemainingData().startsWith('\n') || m_streamParser->readRemainingData().startsWith("\r\n"))
+      m_streamParser->readUntilCommandEnd(); //just eat the ending newline
   }
 }
 
+
 void AkonadiConnection::writeOut( const QByteArray &data )
 {
     QByteArray block = data + "\r\n";
@@ -205,7 +211,7 @@ void AkonadiConnection::slotConnectionStateChange( ConnectionState state )
     }
 }
 
-qint64 Akonadi::AkonadiConnection::selectedCollection( ) const
+qint64 Akonadi::AkonadiConnection::selectedCollectionId( ) const
 {
     return m_selectedConnection;
 }
@@ -215,9 +221,9 @@ void Akonadi::AkonadiConnection::setSelectedCollection( qint64 collection )
     m_selectedConnection = collection;
 }
 
-const Location Akonadi::AkonadiConnection::selectedLocation()
+const Collection Akonadi::AkonadiConnection::selectedCollection()
 {
-  return Location::retrieveById( selectedCollection() );
+  return Collection::retrieveById( selectedCollectionId() );
 }
 
 void Akonadi::AkonadiConnection::addStatusMessage( const QByteArray& msg )
@@ -250,4 +256,24 @@ QByteArray AkonadiConnection::sessionId() const
   return m_sessionId;
 }
 
+Resource Akonadi::AkonadiConnection::resourceContext() const
+{
+  return m_resourceContext;
+}
+
+void AkonadiConnection::setResourceContext(const Resource& res)
+{
+  m_resourceContext = res;
+}
+
+bool AkonadiConnection::isOwnerResource(const PimItem& item) const
+{
+  if ( resourceContext().isValid() )
+    return true;
+  // fallback for older resources
+  if ( sessionId() == item.collection().resource().name().toUtf8() )
+    return true;
+  return false;
+}
+
 #include "akonadiconnection.moc"
