@@ -22,10 +22,12 @@
 #include <QtCore/QDebug>
 #include <QtCore/QLatin1String>
 
-#include "../libs/imapset_p.h"
+#include "libs/imapset_p.h"
+#include "libs/protocol_p.h"
 
 #include "akonadiconnection.h"
 #include "response.h"
+#include "scope.h"
 #include "handler/akappend.h"
 #include "handler/aklist.h"
 #include "handler/append.h"
@@ -42,22 +44,26 @@
 #include "handler/logout.h"
 #include "handler/modify.h"
 #include "handler/noop.h"
+#include "handler/remove.h"
 #include "handler/rename.h"
+#include "handler/resourceselect.h"
 #include "handler/searchpersistent.h"
 #include "handler/select.h"
 #include "handler/subscribe.h"
 #include "handler/status.h"
 #include "handler/store.h"
 #include "handler/transaction.h"
-#include "uid.h"
+#include "handler/uid.h"
 
 #include "storage/querybuilder.h"
+#include "imapstreamparser.h"
 
 using namespace Akonadi;
 
 Handler::Handler()
     :QObject()
     , m_connection( 0 )
+    , m_streamParser( 0 )
 {
 }
 
@@ -66,18 +72,6 @@ Handler::~Handler()
 {
 }
 
-bool Handler::handleLine( const QByteArray & command )
-{
-    Response response;
-    response.setError();
-    response.setTag( m_tag );
-    response.setString( "Unrecognized command: " + command );
-
-    emit responseAvailable( response );
-    deleteLater();
-    return true;
-}
-
 Handler * Handler::findHandlerForCommandNonAuthenticated( const QByteArray & command )
 {
     // allowed are LOGIN and AUTHENTICATE
@@ -117,17 +111,19 @@ Handler * Handler::findHandlerForCommandAuthenticated( const QByteArray & comman
     if ( command == "LIST" )
         return new List();
     if ( command == "SELECT" )
-        return new Select();
+        return new Select( Scope::None );
     if ( command == "SEARCH_STORE" )
         return new SearchPersistent();
     if ( command == "NOOP" )
         return new Noop();
     if ( command == "FETCH" )
-        return new Fetch();
+        return new Fetch( Scope::None );
     if ( command == "EXPUNGE" )
         return new Expunge();
-    if ( command == "UID" )
-        return new Uid();
+    if ( command == AKONADI_CMD_UID )
+        return new Uid( Scope::Uid );
+    if ( command == AKONADI_CMD_RID )
+        return new Uid( Scope::Rid );
     if ( command == "STORE" )
         return new Store();
     if ( command == "STATUS" )
@@ -138,14 +134,22 @@ Handler * Handler::findHandlerForCommandAuthenticated( const QByteArray & comman
       return new Modify();
     if ( command == "RENAME" )
       return new Rename();
-    if ( command == "BEGIN" || command == "ROLLBACK" || command == "COMMIT" )
-      return new TransactionHandler();
+    if ( command == "BEGIN" )
+      return new TransactionHandler( TransactionHandler::Begin );
+    if ( command == "ROLLBACK" )
+      return new TransactionHandler( TransactionHandler::Rollback );
+    if ( command == "COMMIT" )
+      return new TransactionHandler( TransactionHandler::Commit );
     if ( command == "X-AKAPPEND" )
       return new AkAppend();
-    if ( command == "X-AKLIST" || command == "X-AKLSUB" )
-      return new AkList();
-    if ( command == "SUBSCRIBE" || command == "UNSUBSCRIBE" )
-      return new Subscribe();
+    if ( command == "X-AKLIST" )
+      return new AkList( Scope::None, false );
+    if ( command == "X-AKLSUB" )
+      return new AkList( Scope::None, true );
+    if ( command == "SUBSCRIBE" )
+      return new Subscribe( true );
+    if ( command == "UNSUBSCRIBE" )
+      return new Subscribe( false );
     if ( command == "COPY" )
       return new Copy();
     if ( command == "COLCOPY" )
@@ -154,6 +158,10 @@ Handler * Handler::findHandlerForCommandAuthenticated( const QByteArray & comman
       return new Link( true );
     if ( command == "UNLINK" )
       return new Link( false );
+    if ( command == AKONADI_CMD_RESOURCESELECT )
+      return new ResourceSelect();
+    if ( command == AKONADI_CMD_ITEMDELETE )
+      return new Remove( Scope::None );
 
     return 0;
 }
@@ -165,7 +173,7 @@ void Akonadi::Handler::setConnection( AkonadiConnection* connection )
 }
 
 
-AkonadiConnection* Akonadi::Handler::connection()
+AkonadiConnection* Akonadi::Handler::connection() const
 {
     return m_connection;
 }
@@ -180,7 +188,7 @@ bool Akonadi::Handler::failureResponse( const QString& failureMessage )
     response.setString( failureMessage );
     emit responseAvailable( response );
     deleteLater();
-    return true;
+    return false;
 }
 
 bool Akonadi::Handler::failureResponse( const QByteArray &failureMessage )
@@ -204,42 +212,22 @@ bool Handler::successResponse(const char * successMessage)
   return true;
 }
 
-void Handler::imapSetToQuery(const ImapSet & set, bool isUid, QueryBuilder & qb)
+void Handler::setStreamParser( ImapStreamParser *parser )
 {
-  Query::Condition cond( Query::Or );
-  foreach ( const ImapInterval i, set.intervals() ) {
-    if ( i.hasDefinedBegin() && i.hasDefinedEnd() ) {
-      if ( i.size() == 1 ) {
-        cond.addValueCondition( PimItem::idFullColumnName(), Query::Equals, i.begin() );
-      } else {
-        Query::Condition subCond( Query::And );
-        subCond.addValueCondition( PimItem::idFullColumnName(), Query::GreaterOrEqual, i.begin() );
-        subCond.addValueCondition( PimItem::idFullColumnName(), Query::LessOrEqual, i.end() );
-        cond.addCondition( subCond );
-      }
-    } else if ( i.hasDefinedBegin() ) {
-      cond.addValueCondition( PimItem::idFullColumnName(), Query::GreaterOrEqual, i.begin() );
-    } else if ( i.hasDefinedEnd() ) {
-      cond.addValueCondition( PimItem::idFullColumnName(), Query::LessOrEqual, i.end() );
-    }
-  }
-  if ( !cond.isEmpty() )
-    qb.addCondition( cond );
+  m_streamParser = parser;
+}
 
-  if ( !isUid && connection()->selectedCollection() >= 0 ) {
-    const Location loc = connection()->selectedLocation();
-    // FIXME: we probably want to do both paths here in all cases, but that is apparently
-    // non-trivial with SQL
-    if ( loc.resource().name() == QLatin1String("akonadi_search_resource") ||
-         loc.resource().name().contains( QLatin1String("nepomuk") ) ) {
-      qb.addTable( LocationPimItemRelation::tableName() );
-      qb.addValueCondition( LocationPimItemRelation::leftFullColumnName(), Query::Equals, loc.id() );
-      qb.addColumnCondition( LocationPimItemRelation::rightFullColumnName(), Query::Equals,
-                             PimItem::idFullColumnName() );
-    } else {
-      qb.addValueCondition( PimItem::locationIdColumn(), Query::Equals, loc.id() );
-    }
-  }
+bool Handler::parseStream()
+{
+  Response response;
+  response.setError();
+  response.setTag( m_tag );
+  response.setString( "Unrecognized command: "  + m_streamParser->readString() );
+  m_streamParser->readUntilCommandEnd();
+
+  emit responseAvailable( response );
+  deleteLater();
+  return true;
 }
 
 #include "handler.moc"
