@@ -20,101 +20,34 @@
 #include "aklist.h"
 
 #include <QtCore/QDebug>
-#include "../../libs/imapparser_p.h"
 
 #include "storage/datastore.h"
 #include "storage/entity.h"
+#include "storage/selectquerybuilder.h"
 
 #include "akonadiconnection.h"
 #include "response.h"
 #include "handlerhelper.h"
+#include "imapstreamparser.h"
 
 using namespace Akonadi;
 
-AkList::AkList():
+AkList::AkList( Scope::SelectionScope scope, bool onlySubscribed ):
     Handler(),
-    mOnlySubscribed( false )
+    mScope( scope ),
+    mOnlySubscribed( onlySubscribed )
 {}
 
 AkList::~AkList() {}
 
-bool AkList::handleLine(const QByteArray& line )
-{
-  // parse out the reference name and mailbox name
-  int pos = line.indexOf( ' ' ) + 1; // skip tag
-  QByteArray tmp;
-
-  // command
-  pos = ImapParser::parseString( line, tmp, pos );
-  if ( tmp == "X-AKLSUB" )
-    mOnlySubscribed = true;
-
-  qint64 baseCollection;
-  bool ok = false;
-  pos = ImapParser::parseNumber( line, baseCollection, &ok, pos );
-  if ( !ok )
-    return failureResponse( "Invalid base collection" );
-
-  int depth;
-  pos = ImapParser::parseString( line, tmp, pos );
-  if ( tmp.isEmpty() )
-    return failureResponse( "Specify listing depth" );
-  if ( tmp == "INF" )
-    depth = INT_MAX;
-  else
-    depth = tmp.toInt();
-
-  QList<QByteArray> filter;
-  pos = ImapParser::parseParenthesizedList( line, filter, pos );
-
-  for ( int i = 0; i < filter.count() - 1; i += 2 ) {
-    if ( filter.at( i ) == "RESOURCE" ) {
-      mResource = Resource::retrieveByName( QString::fromLatin1( filter.at(i + 1) ) );
-      if ( !mResource.isValid() )
-        return failureResponse( "Unknown resource" );
-    } else
-      return failureResponse( "Invalid filter parameter" );
-  }
-
-  Location::List locations;
-  if ( baseCollection != 0 ) { // not root
-    Location loc = Location::retrieveById( baseCollection );
-    if ( !loc.isValid() )
-      return failureResponse( "Collection " + QByteArray::number( baseCollection ) + " does not exist" );
-    if ( depth == 0 )
-      locations << loc;
-    else {
-      locations << loc.children();
-      --depth;
-    }
-  } else {
-    if ( depth != 0 ) {
-      Location::List list = Location::retrieveFiltered( Location::parentIdColumn(), 0 );
-      locations << list;
-    }
-    --depth;
-  }
-
-  foreach ( const Location &loc, locations )
-    listCollection( loc, depth );
-
-  Response response;
-  response.setSuccess();
-  response.setTag( tag() );
-  response.setString( "List completed" );
-  emit responseAvailable( response );
-  deleteLater();
-  return true;
-}
-
-bool AkList::listCollection(const Location & root, int depth )
+bool AkList::listCollection(const Collection & root, int depth )
 {
   // recursive listing of child collections
   bool childrenFound = false;
   if ( depth > 0 ) {
-    Location::List children = root.children();
-    foreach ( const Location &loc, children ) {
-      if ( listCollection( loc, depth - 1 ) )
+    Collection::List children = root.children();
+    foreach ( const Collection &col, children ) {
+      if ( listCollection( col, depth - 1 ) )
         childrenFound = true;
     }
   }
@@ -125,7 +58,7 @@ bool AkList::listCollection(const Location & root, int depth )
     return false;
 
   // write out collection details
-  Location dummy = root;
+  Collection dummy = root;
   DataStore *db = connection()->storageBackend();
   db->activeCachePolicy( dummy );
   const QByteArray b = HandlerHelper::collectionToByteArray( dummy, hidden );
@@ -137,3 +70,93 @@ bool AkList::listCollection(const Location & root, int depth )
 
   return true;
 }
+
+bool AkList::parseStream()
+{
+  qint64 baseCollection = -1;
+  QString rid;
+  if ( mScope == Scope::None || mScope == Scope::Uid ) {
+    bool ok = false;
+    baseCollection = m_streamParser->readNumber( &ok );
+    if ( !ok )
+      return failureResponse( "Invalid base collection" );
+  } else if ( mScope == Scope::Rid ) {
+    rid = m_streamParser->readUtf8String();
+    if ( rid.isEmpty() )
+      throw HandlerException( "No remote identifier specified" );
+  } else
+    throw HandlerException( "WTF" );
+
+  int depth;
+  const QByteArray tmp = m_streamParser->readString();
+  if ( tmp.isEmpty() )
+    return failureResponse( "Specify listing depth" );
+  if ( tmp == "INF" )
+    depth = INT_MAX;
+  else
+    depth = tmp.toInt();
+
+  QList<QByteArray> filter = m_streamParser->readParenthesizedList();
+
+  for ( int i = 0; i < filter.count() - 1; i += 2 ) {
+    if ( filter.at( i ) == "RESOURCE" ) {
+      mResource = Resource::retrieveByName( QString::fromLatin1( filter.at(i + 1) ) );
+      if ( !mResource.isValid() )
+        return failureResponse( "Unknown resource" );
+    } else
+      return failureResponse( "Invalid filter parameter" );
+  }
+
+  Collection::List collections;
+  if ( baseCollection != 0 ) { // not root
+    Collection col;
+    if ( mScope == Scope::None || mScope == Scope::Uid ) {
+       col = Collection::retrieveById( baseCollection );
+    } else if ( mScope == Scope::Rid ) {
+      SelectQueryBuilder<Collection> qb;
+      qb.addTable( Resource::tableName() );
+      qb.addValueCondition( Collection::remoteIdFullColumnName(), Query::Equals, rid );
+      qb.addColumnCondition( Collection::resourceIdFullColumnName(), Query::Equals, Resource::idFullColumnName() );
+      if ( mResource.isValid() )
+        qb.addValueCondition( Resource::idFullColumnName(), Query::Equals, mResource.id() );
+      else if ( connection()->resourceContext().isValid() )
+        qb.addValueCondition( Resource::idFullColumnName(), Query::Equals, connection()->resourceContext().id() );
+      else
+        throw HandlerException( "Cannot retrieve collection based on remote identifier without a resource context" );
+      if ( !qb.exec() )
+        throw HandlerException( "Unable to retrieve collection for listing" );
+      Collection::List results = qb.result();
+      if ( results.count() != 1 )
+        throw HandlerException( QByteArray::number( results.count() ) + " collections found" );
+      col = results.first();
+    } else
+      throw HandlerException( "WTF" );
+    if ( !col.isValid() )
+      return failureResponse( "Collection " + QByteArray::number( baseCollection ) + " does not exist" );
+    if ( depth == 0 )
+      collections << col;
+    else {
+      collections << col.children();
+      --depth;
+    }
+  } else {
+    if ( depth != 0 ) {
+      Collection::List list = Collection::retrieveFiltered( Collection::parentIdColumn(), 0 );
+      collections << list;
+    }
+    --depth;
+  }
+
+  foreach ( const Collection &col, collections )
+    listCollection( col, depth );
+
+  Response response;
+  response.setSuccess();
+  response.setTag( tag() );
+  response.setString( "List completed" );
+  emit responseAvailable( response );
+  deleteLater();
+  return true;
+}
+
+#include "aklist.moc"
