@@ -20,17 +20,22 @@
 
 #include "append.h"
 #include "response.h"
-#include "../../libs/imapparser_p.h"
 #include "handlerhelper.h"
+#include "libs/imapparser_p.h"
+#include "imapstreamparser.h"
 
 #include "akonadi.h"
 #include "akonadiconnection.h"
 #include "storage/datastore.h"
 #include "storage/entity.h"
 #include "storage/transaction.h"
+#include "storage/parthelper.h"
+#include "storage/dbconfig.h"
 
 #include <QtCore/QDebug>
 #include <QLocale>
+#include <QFile>
+#include <QTemporaryFile>
 
 using namespace Akonadi;
 
@@ -44,67 +49,46 @@ Append::~Append()
 {
 }
 
-bool Akonadi::Append::handleLine(const QByteArray& line )
+bool Append::commit()
 {
-    // Arguments:  mailbox name
-    //        OPTIONAL flag parenthesized list
-    //        OPTIONAL date/time string
-    //        message literal
-    //
-    // Syntax:
-    // append = "APPEND" SP mailbox SP size [SP flag-list] [SP date-time] SP literal
+    bool storeInFile = false;
+    qint64 dataSize = 0;
+    QTemporaryFile tmpFile;
 
-    const int startOfCommand = line.indexOf( ' ' ) + 1;
-    const int startOfMailbox = line.indexOf( ' ', startOfCommand ) + 1;
-
-    const int startOfSize = ImapParser::parseString( line, m_mailbox, startOfMailbox ) + 1;
-
-    QString size;
-    const int startOfFlags = ImapParser::parseString( line, size, startOfSize ) + 1;
-    m_size = size.toLongLong();
-
-    QString data;
-    ImapParser::parseString( line, data, startOfSize);
-    m_size = data.toLongLong();
-
-    // parse optional flag parenthesized list
-    // Syntax:
-    // flag-list      = "(" [flag *(SP flag)] ")"
-    // flag           = "\Answered" / "\Flagged" / "\Deleted" / "\Seen" /
-    //                  "\Draft" / flag-keyword / flag-extension
-    //                    ; Does not include "\Recent"
-    // flag-extension = "\" atom
-    // flag-keyword   = atom
-    int startOfDateTime = startOfFlags;
-    if ( line[startOfFlags] == '(' ) {
-        startOfDateTime = ImapParser::parseParenthesizedList( line, m_flags, startOfFlags ) + 1;
-    }
-
-    // parse optional date/time string
-    int startOfLiteral = startOfDateTime;
-    if ( line[startOfDateTime] == '"' ) {
-        startOfLiteral = ImapParser::parseDateTime( line, m_dateTime, startOfDateTime );
-        m_dateTime = m_dateTime.toUTC();
-        // FIXME Should we return an error if m_dateTime is invalid?
+    //need to read out the data before the below code to avoid hangs, probably at some place the eventloop is reentered
+    //and more data is read into the stream and that causes the problems.
+    if ( m_streamParser->hasLiteral() ) {
+      dataSize = m_streamParser->remainingLiteralSize();
+      storeInFile = DbConfig::useExternalPayloadFile() && dataSize > DbConfig::sizeThreshold();
+      if ( storeInFile ) {
+        if ( !tmpFile.open() ) {
+          storeInFile =  false;
+        }
+      }
+      while ( !m_streamParser->atLiteralEnd() ) {
+        if ( !storeInFile ) {
+          m_data += m_streamParser->readLiteralPart();
+        } else
+        {
+          m_data = m_streamParser->readLiteralPart();
+          tmpFile.write( m_data );
+        }
+      }
+      if ( storeInFile ) {
+        tmpFile.close();
+        m_data = "";
+      }
     } else {
-        // if date/time is not given then it will be set to the current date/time
-        // converted to UTC.
-        m_dateTime = QDateTime::currentDateTime().toUTC();
+      m_data = m_streamParser->readString();
     }
 
-    ImapParser::parseString( line, m_data, startOfLiteral );
-    return commit();
-}
-
-bool Akonadi::Append::commit()
-{
     Response response;
 
     DataStore *db = connection()->storageBackend();
     Transaction transaction( db );
 
-    Location l = HandlerHelper::collectionFromIdOrName( m_mailbox );
-    if ( !l.isValid() )
+    Collection col = HandlerHelper::collectionFromIdOrName( m_mailbox );
+    if ( !col.isValid() )
       return failureResponse( "Unknown collection." );
 
     QByteArray mt;
@@ -126,7 +110,10 @@ bool Akonadi::Append::commit()
     if ( mt.isEmpty() ) mt = "message/rfc822";
     MimeType mimeType = MimeType::retrieveByName( QString::fromLatin1( mt ) );
     if ( !mimeType.isValid() ) {
-      return failureResponse( QString::fromLatin1( "Unknown mime type '%1'.").arg( QString::fromLatin1( mt ) ) );
+      MimeType m( QString::fromLatin1( mt ) );
+      if ( !m.insert() )
+        return failureResponse( QString::fromLatin1( "Unable to create mimetype '%1'.").arg( QString::fromLatin1( mt ) ) );
+      mimeType = m;
     }
 
     PimItem item;
@@ -139,19 +126,38 @@ bool Akonadi::Append::commit()
     part.setName( QLatin1String( "PLD:RFC822" ) );
     part.setData( m_data );
     part.setPimItemId( item.id() );
+    if (storeInFile) {
+      part.setDatasize( dataSize );
+      part.setExternal( true );//force external storage
+    }
+
     QList<Part> parts;
     parts.append( part );
 
-    bool ok = db->appendPimItem( parts, mimeType, l, m_dateTime, remote_id, item );
+    bool ok = db->appendPimItem( parts, mimeType, col, m_dateTime, remote_id, item );
     response.setTag( tag() );
     if ( !ok ) {
         return failureResponse( "Append failed" );
     }
 
     // set message flags
-    if ( !db->appendItemFlags( item, flags, false, l ) )
+    if ( !db->appendItemFlags( item, flags, false, col ) )
       return failureResponse( "Unable to append item flags." );
 
+    if (storeInFile) {
+      part.setExternal( true );
+      //the new item was just created and the transaction is not yet committed, so delete + overwrite should be safe, as no
+      //client knows about the item yet
+      QString fileName = QString::fromUtf8( parts[0].data() );
+      QFile f( fileName );
+      if ( !f.remove() ) {
+        return failureResponse( "Unable to remove item part file" );
+      }
+      if ( !tmpFile.copy( fileName ) ) {
+        return failureResponse( "Unable to copy item part data from the temporary file" );
+      }
+    }
+
     // TODO if the mailbox is currently selected, the normal new message
     //      actions SHOULD occur.  Specifically, the server SHOULD notify the
     //      client immediately via an untagged EXISTS response.
@@ -178,3 +184,46 @@ bool Akonadi::Append::commit()
 
     return true;
 }
+
+bool Append::parseStream()
+{
+    // Arguments:  mailbox name
+    //        OPTIONAL flag parenthesized list
+    //        OPTIONAL date/time string
+    //        message literal
+  //
+    // Syntax:
+    // append = "APPEND" SP mailbox SP size [SP flag-list] [SP date-time] SP literal
+
+  m_mailbox = m_streamParser->readString();
+
+  bool ok = false;
+  m_size = m_streamParser->readNumber( &ok );
+
+    // parse optional flag parenthesized list
+    // Syntax:
+    // flag-list      = "(" [flag *(SP flag)] ")"
+  // flag           = "\Answered" / "\Flagged" / "\Deleted" / "\Seen" /
+    //                  "\Draft" / flag-keyword / flag-extension
+    //                    ; Does not include "\Recent"
+    // flag-extension = "\" atom
+    // flag-keyword   = atom
+  if ( m_streamParser->hasList() ) {
+    m_flags = m_streamParser->readParenthesizedList();
+  }
+
+    // parse optional date/time string
+  if ( m_streamParser->hasDateTime() ) {
+    m_dateTime = m_streamParser->readDateTime();
+    m_dateTime = m_dateTime.toUTC();
+        // FIXME Should we return an error if m_dateTime is invalid?
+  } else {
+        // if date/time is not given then it will be set to the current date/time
+        // converted to UTC.
+    m_dateTime = QDateTime::currentDateTime().toUTC();
+  }
+
+  return commit();
+}
+
+#include "append.moc"
