@@ -21,33 +21,58 @@
 
 #include "akonadiconnection.h"
 #include "handlerhelper.h"
-#include "../../libs/imapparser_p.h"
-#include "../../libs/imapset_p.h"
 
 #include "storage/datastore.h"
+#include "storage/itemqueryhelper.h"
+#include "storage/itemretriever.h"
 #include "storage/selectquerybuilder.h"
 #include "storage/transaction.h"
+#include "storage/parthelper.h"
+
+#include "libs/imapset_p.h"
+#include "imapstreamparser.h"
 
 using namespace Akonadi;
 
-bool Copy::handleLine(const QByteArray & line)
+bool Copy::copyItem(const PimItem & item, const Collection & target)
 {
-  QByteArray tmp;
-  int pos = ImapParser::parseString( line, tmp ); // skip tag
-  pos = ImapParser::parseString( line, tmp, pos ); // skip command
+  qDebug() << "Copy::copyItem";
+  DataStore *store = connection()->storageBackend();
+  PimItem newItem = item;
+  newItem.setId( -1 );
+  newItem.setRev( 0 );
+  newItem.setSize( 0 );
+  newItem.setDatetime( QDateTime::currentDateTime() );
+  newItem.setRemoteId( QByteArray() );
+  newItem.setCollectionId( target.id() );
+  Part::List parts;
+  foreach ( const Part &part, item.parts() ) {
+    Part newPart( part );
+    newPart.setData(PartHelper::translateData(newPart.id(), newPart.data(), part.external()));
+    newPart.setPimItemId( -1 );
+    parts << newPart;
+  }
+  return store->appendPimItem( parts, item.mimeType(), target, QDateTime::currentDateTime(), QByteArray(), newItem );
+}
 
-  ImapSet set;
-  pos = ImapParser::parseSequenceSet( line, set, pos );
+bool Copy::parseStream()
+{
+  ImapSet set = m_streamParser->readSequenceSet();
   if ( set.isEmpty() )
     return failureResponse( "No items specified" );
 
-  ImapParser::parseString( line, tmp, pos );
-  const Location loc = HandlerHelper::collectionFromIdOrName( tmp );
-  if ( !loc.isValid() )
+  ItemRetriever retriever( connection() );
+  retriever.setItemSet( set );
+  retriever.setRetrieveFullPayload( true );
+  retriever.exec();
+
+  const QByteArray tmp = m_streamParser->readString();
+  const Collection col = HandlerHelper::collectionFromIdOrName( tmp );
+  if ( !col.isValid() )
     return failureResponse( "No valid target specified" );
 
   SelectQueryBuilder<PimItem> qb;
-  imapSetToQuery( set, true, qb );
+  ItemQueryHelper::itemSetToQuery( set, qb );
   if ( !qb.exec() )
     return failureResponse( "Unable to retrieve items" );
   PimItem::List items = qb.result();
@@ -56,7 +81,7 @@ bool Copy::handleLine(const QByteArray & line)
   Transaction transaction( store );
 
   foreach ( const PimItem &item, items ) {
-    if ( !copyItem( item, loc ) )
+    if ( !copyItem( item, col ) )
       return failureResponse( "Unable to copy item" );
   }
 
@@ -66,21 +91,4 @@ bool Copy::handleLine(const QByteArray & line)
   return successResponse( "COPY complete" );
 }
 
-bool Copy::copyItem(const PimItem & item, const Location & target)
-{
-  DataStore *store = connection()->storageBackend();
-  PimItem newItem = item;
-  newItem.setId( -1 );
-  newItem.setRev( 0 );
-  newItem.setSize( 0 );
-  newItem.setDatetime( QDateTime::currentDateTime() );
-  newItem.setRemoteId( QByteArray() );
-  newItem.setLocationId( target.id() );
-  Part::List parts;
-  foreach ( const Part &part, item.parts() ) {
-    Part newPart( part );
-    newPart.setPimItemId( -1 );
-    parts << newPart;
-  }
-  return store->appendPimItem( parts, item.mimeType(), target, QDateTime::currentDateTime(), QByteArray(), newItem );
-}
+#include "copy.moc"
