@@ -33,13 +33,14 @@ namespace Akonadi {
 */
 class Delete : public Handler
 {
+  Q_OBJECT
   public:
     Delete();
     ~Delete();
-    virtual bool handleLine( const QByteArray &line );
+    bool parseStream();
 
   private:
-    bool deleteRecursive( Location &loc );
+    bool deleteRecursive( Collection &col );
 
 };
 
