@@ -22,8 +22,8 @@
 
 #include <handler.h>
 
-#include "../../libs/imapset_p.h"
-
+#include "scope.h"
+#include "libs/imapset_p.h"
 #include "storage/datastore.h"
 
 namespace Akonadi {
@@ -38,12 +38,11 @@ class QueryBuilder;
  */
 class Fetch : public Handler
 {
+  Q_OBJECT
   public:
-    Fetch();
+    Fetch( Scope::SelectionScope scope );
 
-    ~Fetch();
-
-    bool handleLine(const QByteArray& line);
+    bool parseStream();
 
   private:
     void parseCommand( const QByteArray &line );
@@ -52,15 +51,21 @@ class Fetch : public Handler
     void buildItemQuery();
     QueryBuilder buildPartQuery( const QStringList &partList, bool allPayload, bool allAttrs );
     void retrieveMissingPayloads( const QStringList &payloadList );
+    void parseCommandStream();
+    void addQueryConditions( QueryBuilder &qb );
 
   private:
     QueryBuilder mItemQuery;
     ImapSet mSet;
+    QString mRid;
     QList<QByteArray> mRequestedParts;
-    bool mIsUidFetch;
+    Scope::SelectionScope mScope;
     bool mCacheOnly;
     bool mFullPayload;
     bool mAllAttrs;
+    bool mSizeRequested;
+    bool mMTimeRequested;
+    bool mExternalPayloadSupported;
 };
 
 }
