@@ -19,14 +19,14 @@
 
 #include "resourceselect.h"
 
-#include <connection.h>
+#include <akonadiconnection.h>
 #include <entities.h>
 #include <imapstreamparser.h>
 
-using namespace Akonadi::Server;
+using namespace Akonadi;
 
-ResourceSelect::ResourceSelect()
-  : Handler()
+ResourceSelect::ResourceSelect() :
+  Handler()
 {
 }
 
@@ -34,16 +34,19 @@ bool ResourceSelect::parseStream()
 {
   const QString resourceName = m_streamParser->readUtf8String();
   if ( resourceName.isEmpty() ) {
-    connection()->context()->setResource( Resource() );
+    connection()->setResourceContext( Resource() );
+    deleteLater();
     return successResponse( "Resource deselected" );
   }
 
   const Resource res = Resource::retrieveByName( resourceName );
-  if ( !res.isValid() ) {
+  if ( !res.isValid() )
     throw HandlerException( resourceName.toUtf8() + " is not a valid resource identifier" );
-  }
 
-  connection()->context()->setResource( res );
+  connection()->setResourceContext( res );
 
+  deleteLater();
   return successResponse( resourceName.toUtf8() + " selected" );
 }
+
+#include "resourceselect.moc"
