@@ -23,6 +23,7 @@
 #include <handler.h>
 
 #include "storage/entity.h"
+#include "libs/imapset_p.h"
 
 namespace Akonadi {
 
@@ -35,11 +36,12 @@ class PimItem;
  */
 class Store : public Handler
 {
+  Q_OBJECT
+
   public:
-    Store();
+    Store( bool isUid = false );
     ~Store();
 
-    bool handleLine(const QByteArray& line);
 
     enum Operation
     {
@@ -48,12 +50,23 @@ class Store : public Handler
       Delete
     };
 
+    bool parseStream();
 
   private:
+    void parseCommand( const QByteArray &line );
+    void parseCommandStream();
+
     bool replaceFlags( const PimItem &item, const QList<QByteArray> &flags );
     bool addFlags( const PimItem &item, const QList<QByteArray> &flags );
     bool deleteFlags( const PimItem &item, const QList<QByteArray> &flags );
-    bool deleteParts( const PimItem &item, const QList<QByteArray> &parts );
+    void sendPimItemResponse( const PimItem &pimItem );
+
+  private:
+    ImapSet mItemSet;
+    int mPos;
+    qint64 mPreviousRevision;
+    qint64 mSize;
+    bool mUidStore;
 };
 
 }
