@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2006 Volker Krause <vkrause@kde.org>
+    Copyright (c) 2006 Volker Krause <volker.krause@rwth-aachen.de>
 
     This library is free software; you can redistribute it and/or modify it
     under the terms of the GNU Library General Public License as published by
@@ -19,47 +19,43 @@
 
 #include "transaction.h"
 #include "storage/datastore.h"
-#include "connection.h"
+#include "akonadiconnection.h"
 #include "response.h"
 #include "imapstreamparser.h"
 
 #include <QMetaEnum>
 
-using namespace Akonadi::Server;
-
-TransactionHandler::TransactionHandler( Mode mode )
-  : mMode( mode )
+Akonadi::TransactionHandler::TransactionHandler( Mode mode ) :
+  mMode( mode )
 {
 }
 
-bool TransactionHandler::parseStream()
+bool Akonadi::TransactionHandler::parseStream()
 {
   DataStore *store = connection()->storageBackend();
 
   if ( mMode == Begin ) {
-    if ( !store->beginTransaction() ) {
+    if ( !store->beginTransaction() )
       return failureResponse( "Unable to begin transaction." );
-    }
   }
 
   if ( mMode == Rollback ) {
-    if ( !store->inTransaction() ) {
+    if ( !store->inTransaction() )
       return failureResponse( "There is no transaction in progress." );
-    }
-    if ( !store->rollbackTransaction() ) {
+    if ( !store->rollbackTransaction() )
       return failureResponse( "Unable to roll back transaction." );
-    }
   }
 
   if ( mMode == Commit ) {
-    if ( !store->inTransaction() ) {
+    if ( !store->inTransaction() )
       return failureResponse( "There is no transaction in progress." );
-    }
-    if ( !store->commitTransaction() ) {
+    if ( !store->commitTransaction() )
       return failureResponse( "Unable to commit transaction." );
-    }
   }
 
+  deleteLater();
   const QMetaEnum me = metaObject()->enumerator( metaObject()->indexOfEnumerator( "Mode" ) );
   return successResponse( me.valueToKey( mMode ) + QByteArray( " completed" ) );
 }
+
+#include "transaction.moc"
