@@ -31,11 +31,21 @@ namespace Akonadi {
 */
 class TransactionHandler : public Handler
 {
-  public:
-    TransactionHandler();
-    ~TransactionHandler();
+  Q_OBJECT
+  Q_ENUMS( Mode )
 
-    bool handleLine( const QByteArray &line );
+  public:
+    enum Mode {
+      Begin,
+      Commit,
+      Rollback
+    };
+
+    TransactionHandler( Mode mode );
+    bool parseStream();
+
+  private:
+    Mode mMode;
 };
 
 }
