@@ -50,6 +50,22 @@ class DbConfigStatic
           akFatal() << "No usable database driver found.";
       }
 
+      mSizeThreshold = 4096;
+      QVariant v = settings.value( QLatin1String("General/SizeThreshold"), mSizeThreshold );
+      if ( v.canConvert<qint64>() )
+      {
+        mSizeThreshold = v.value<qint64>();
+      } else
+      {
+        mSizeThreshold = 0;
+      }
+
+      if ( mSizeThreshold < 0 )
+        mSizeThreshold = 0;
+
+      mUseExternalPayloadFile = false;
+      mUseExternalPayloadFile = settings.value( QLatin1String("General/ExternalPayload"), mUseExternalPayloadFile ).toBool();
+
       // determine default settings depending on the driver
       QString defaultDbName;
       QString defaultOptions;
@@ -106,6 +122,8 @@ class DbConfigStatic
 
       // store back the default values
       settings.setValue( QLatin1String( "General/Driver" ), mDriverName );
+      settings.setValue( QLatin1String( "General/SizeThreshold" ), mSizeThreshold );
+      settings.setValue( QLatin1String( "General/ExternalPayload" ), mUseExternalPayloadFile );
       settings.beginGroup( mDriverName );
       settings.setValue( QLatin1String( "Name" ), mName );
       settings.setValue( QLatin1String( "User" ), mUserName );
@@ -188,6 +206,8 @@ class DbConfigStatic
     QString mConnectionOptions;
     QString mServerPath;
     bool mInternalServer;
+    bool mUseExternalPayloadFile;
+    qint64 mSizeThreshold;
 };
 
 Q_GLOBAL_STATIC( DbConfigStatic, sInstance )
@@ -234,3 +254,13 @@ QString DbConfig::databaseName()
   return sInstance()->mName;
 }
 
+qint64 DbConfig::sizeThreshold()
+{
+  return sInstance()->mSizeThreshold;
+}
+
+bool DbConfig::useExternalPayloadFile()
+{
+  return sInstance()->mUseExternalPayloadFile;
+}
+
